﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Bespoke.Common.Osc;
using System.Net;

namespace ZOIL.Library.Behaviors
{
    public class GenericOSCHandler
    {
        private static OscServer _server = null;

        private static short DEFAULT_PORT = 4411;
        private static IPAddress DEFAULT_IP = IPAddress.Parse("224.5.6.7");
        private static String DEFAULT_OSC_ADDRESS = "/zoil";

        public static IObservable<OscMessageReceivedEventArgs> OscMessageAsObservable()
        {
            return OscMessageAsObservable(DEFAULT_PORT, DEFAULT_IP, DEFAULT_OSC_ADDRESS);
        }

        public static IObservable<OscMessageReceivedEventArgs> OscMessageAsObservable(int port, IPAddress ip, String oscAddress)
        {
            if (!oscAddress.StartsWith("/"))
                throw new ArgumentException("oscAddress must start with forward slash (/)", "oscAddress");

            if(_server == null)
            {
                _server = new OscServer(port, ip);
                _server.Start();
            }

            if(!_server.RegisteredMethods.Contains(oscAddress))
                _server.RegisterMethod(oscAddress);

            var message = Observable.FromEvent<OscMessageReceivedEventArgs>(_server, "MessageReceived");

            return message.Where(evt => evt.EventArgs.Message.Address == oscAddress).Select(evt => evt.EventArgs);
        }
    }
}
