﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Input;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Media.Media3D;
using ZOIL.Framework.Presentation;
using ZOIL.Framework.Core;
using Microsoft.Surface.Presentation;

namespace ZOIL.Library.Behaviors
{
    public static class ZObjectDragDropBehavior
    {
        #region events

        
        public delegate void DragDropEventHandler(object sender, ZObjectDragDropEventArgs args);

        public static readonly RoutedEvent DragStartedEvent = EventManager.RegisterRoutedEvent("DragStarted", RoutingStrategy.Bubble, typeof(DragDropEventHandler), typeof(ZObjectDragDropBehavior));
        public static readonly RoutedEvent DragFinishedEvent = EventManager.RegisterRoutedEvent("DragFinished", RoutingStrategy.Bubble, typeof(DragDropEventHandler), typeof(ZObjectDragDropBehavior));
        public static readonly RoutedEvent DragDeltaEvent = EventManager.RegisterRoutedEvent("DragDelta", RoutingStrategy.Bubble, typeof(DragDropEventHandler), typeof(ZObjectDragDropBehavior));

        
        #endregion

        #region attached properties

        #region IsDragging attached property (for reading out current dragging status)

        public static DependencyProperty IsDraggingProperty = DependencyProperty.RegisterAttached("IsDragging",
                    typeof(bool),
                    typeof(ZObjectDragDropBehavior),
                    new FrameworkPropertyMetadata(false, new PropertyChangedCallback(ZObjectDragDropBehavior.IsDraggingChanged)));

        private static void IsDraggingChanged(DependencyObject target, DependencyPropertyChangedEventArgs e)
        {

        }

        public static void SetIsDragging(DependencyObject target, bool value)
        {
            target.SetValue(ZObjectDragDropBehavior.IsDraggingProperty, value);
        }

        public static bool GetIsDragging(DependencyObject d)
        {
            return (bool)d.GetValue(ZObjectDragDropBehavior.IsDraggingProperty);
        }


        #endregion

        #region IsDraggable attached property (for setting drag & drop behavior active or unactive)

        public static DependencyProperty IsDraggableProperty = DependencyProperty.RegisterAttached("IsDraggable",
                    typeof(bool),
                    typeof(ZObjectDragDropBehavior),
                    new FrameworkPropertyMetadata(false, new PropertyChangedCallback(ZObjectDragDropBehavior.IsDraggableChanged)));

        private static void IsDraggableChanged(DependencyObject target, DependencyPropertyChangedEventArgs e)
        {


            //if((bool)e.NewValue)
            //{
            //    if(target is FrameworkElement)
            //    {
            //        (target as FrameworkElement).Cursor = Cursors.Hand;
            //    }
            //}

            //following section forwards a change of IsDraggableChanged property to method ZBehavior.HandleChange, 
            //which then will (un)register event handlers according to new value of property.

            // TODO: do this for all possible devices (e.g. Mouse, Surface, Wii) 
            // TODO: there should be a more elegant solution than to just go through all devices without much thought. Ideally devices become dynamically (un)loadable
            ZBehavior.HandleChange(e.OldValue, e.NewValue, target as ZComponent, Mouse.MouseDownEvent, new MouseButtonEventHandler(OnMouseDown));
            ZBehavior.HandleChange(e.OldValue, e.NewValue, target as ZComponent, Mouse.MouseUpEvent, new MouseButtonEventHandler(OnMouseUp));
            ZBehavior.HandleChange(e.OldValue, e.NewValue, target as ZComponent, Mouse.MouseMoveEvent, new MouseEventHandler(OnMouseMove));

            //Wii


            //Surface
            ZBehavior.HandleChange(e.OldValue, e.NewValue, target as UIElement, Contacts.ContactDownEvent, new ContactEventHandler(ZObjectSurfaceHandler.OnContactDown));
            ZBehavior.HandleChange(e.OldValue, e.NewValue, target as UIElement, Contacts.ContactUpEvent, new ContactEventHandler(ZObjectSurfaceHandler.OnContactUp));
            ZBehavior.HandleChange(e.OldValue, e.NewValue, target as UIElement, Contacts.ContactTapGestureEvent, new ContactEventHandler(ZObjectSurfaceHandler.OnContactTapGesture));
            ZBehavior.HandleChange(e.OldValue, e.NewValue, target as UIElement, Contacts.ContactHoldGestureEvent, new ContactEventHandler(ZObjectSurfaceHandler.OnContactHold));

            //Windows 7
            //ZBehavior.HandleChange(e.OldValue, e.NewValue, target as UIElement, UIElement.StylusDownEvent, new StylusDownEventHandler(ZObjectWindows7Handler.OnStylusDown));
            //ZBehavior.HandleChange(e.OldValue, e.NewValue, target as UIElement, UIElement.StylusMoveEvent, new StylusEventHandler(ZObjectWindows7Handler.OnStylusMove));
            //ZBehavior.HandleChange(e.OldValue, e.NewValue, target as UIElement, UIElement.StylusUpEvent, new StylusEventHandler(ZObjectWindows7Handler.OnStylusUp));
            //ZBehavior.HandleChange(e.OldValue, e.NewValue, target as UIElement, UIElement.StylusSystemGestureEvent, new StylusSystemGestureEventHandler(ZObjectWindows7Handler.OnStylusSystemGesture));

            //ZBehavior.HandleChange(e.OldValue, e.NewValue, target as UIElement, ZObjectDragDropBehavior.DragFinishedEvent, new DragDropEventHandler(OnDragFinishedCheckDropTarget));
        }

        private static void OnDragFinishedCheckDropTarget(object sender, ZObjectDragDropEventArgs e)
        {
            FrameworkElement uie = sender as FrameworkElement;
            
            var possibleDropTargets = ZHitTestHelper.GetElementsInGeometry(new RectangleGeometry(new Rect(Canvas.GetLeft(uie), Canvas.GetTop(uie), uie.Width, uie.Height)), ZTreeHelper.TryFindParent<ZInformationLandscape>(uie));

            foreach (UIElement element in possibleDropTargets)
            {

            }

        }

        private static bool IsPossibleDropTarget(Type type)
        {
            return _dropRegistry.Values.Select(behavior => behavior.Type).Contains(type);
        }

        private static Dictionary<UIElement, ZObjectDragInformation> ElementsBeingDragged = new Dictionary<UIElement, ZObjectDragInformation>();

         //---------------------------------------------------------
         //device specific implementation of dragging behavior:
         //these manage the currently dragged objects and raise the events to issue commands on UIElement

        #region mouse

        static void OnMouseDown(object sender, MouseButtonEventArgs e)
        {
            UIElement element = sender as UIElement;

            if (e.StylusDevice != null)
                return;

            if(ElementsBeingDragged.ContainsKey(element))
            {
                return;
            }

            ZObjectDragInformation info = new ZObjectDragInformation(e.GetPosition(null));

            info.ElementBeingDragged = ZBehavior.FindCanvasChild(element);

            //set ZIndex of ElementBeingDragged
            if (info.ElementBeingDragged is ZComponent)
            {
                Panel.SetZIndex(info.ElementBeingDragged, ZComponent.ZIndexCounter++);
            }

            info.OriginalFrameworkElement = (e.Source as FrameworkElement != null) ? e.Source as FrameworkElement : info.ElementBeingDragged as FrameworkElement;
            info.Bottom = Canvas.GetBottom(info.ElementBeingDragged);
            info.Top = Canvas.GetTop(info.ElementBeingDragged);
            info.Left = Canvas.GetLeft(info.ElementBeingDragged);
            info.Right = Canvas.GetRight(info.ElementBeingDragged);
            info.OriginalPositionOnUIElement = e.GetPosition(element);
            // TODO: replace using TransformToAncestor()
            info.ParentLandscapes = ZTreeHelper.TryFindParents<ZInformationLandscape>(element);

            ElementsBeingDragged[element] = info;

            element.CaptureMouse();
            e.Handled = true;
        }

        static void OnMouseUp(object sender, MouseButtonEventArgs e)
        {
            UIElement element = sender as UIElement;

            if(ElementsBeingDragged.ContainsKey(sender as UIElement))
            {
                ZObjectDragInformation info = ElementsBeingDragged[sender as UIElement];
                if (!ZObjectDragDropBehavior.GetIsDragging(element))
                {
                    //apply click zoom here, because we do not drag
                    FrameworkElement zoomTarget = ZTreeHelper.GetClosestZoomTarget(info.ElementBeingDragged as FrameworkElement, info.OriginalFrameworkElement);
                    if (zoomTarget != null)
                    {
                        info.ParentLandscapes.First().ZoomOnto(zoomTarget, false, true);
                        zoomTarget.RaiseEvent(new RoutedEventArgs(ZInformationLandscape.ClickZoomEvent, zoomTarget));
                    }
                }

                ElementsBeingDragged.Remove(sender as UIElement);
            }
            element.ReleaseMouseCapture();
            e.Handled = true;

            //Raise DragFinishedEvent and set IsDragging to false
            ZObjectDragDropEventArgs args = new ZObjectDragDropEventArgs();
            args.RoutedEvent = ZObjectDragDropBehavior.DragFinishedEvent;
            args.Source = element;
            args.CurrentDragPoint = e.GetPosition(null);
            element.RaiseEvent(args);

            //hack, to raise the event on the clipboard (JS)
            //removed hack ! since event has been raised two times which causes unforseen behavior (MZ)
            //FrameworkElement ui = Mouse.DirectlyOver as FrameworkElement;
            //if(ui != null) ui.RaiseEvent(args);

            ZObjectDragDropBehavior.SetIsDragging(element, false);
        }

        static void OnMouseMove(object sender, MouseEventArgs e)
        {
            UIElement element = sender as UIElement;
            if(ElementsBeingDragged.ContainsKey(element))
            {
                ZObjectDragInformation info = ElementsBeingDragged[element];
                Point currentPoint = e.GetPosition(null);

                double scale = info.getScaleFactor();

                Vector diff = Point.Subtract(currentPoint, info.OriginalPositionOnWindow);

                //double xdiff = (currentPoint.X - info.OriginalMousePositionOnWindow.X);
                //double ydiff = (currentPoint.Y - info.OriginalMousePositionOnWindow.Y);

                if(diff.Length > 5)
                {
                    if (!ZObjectDragDropBehavior.GetIsDragging(element))
                    {
                        
                        ZObjectDragDropBehavior.SetIsDragging(element, true);

                        //Raise OnDragStartedEvent and set IsDragging to true
                        ZObjectDragDropEventArgs args = new ZObjectDragDropEventArgs();
                        args.RoutedEvent = ZObjectDragDropBehavior.DragStartedEvent;
                        args.Source = element;
                        args.CurrentDragPoint = e.GetPosition(null);
                        element.RaiseEvent(args);
                        //element.SetValue(ZObjectDragDropBehavior.IsDraggingProperty, true);
                    }
                    else
                    {
                        //Raise DragDeltaEvent
                        ZObjectDragDropEventArgs args = new ZObjectDragDropEventArgs();
                        args.CurrentDragPoint = currentPoint;
                        args.RoutedEvent = ZObjectDragDropBehavior.DragDeltaEvent;
                        args.Source = element;
                        element.RaiseEvent(args);
                    }
                }

                double newX = info.Left + diff.X / scale;
                double newY = info.Top + diff.Y / scale;

                Canvas.SetLeft(info.ElementBeingDragged, newX);
                Canvas.SetTop(info.ElementBeingDragged, newY);

                e.Handled = true;
            }
        }

        //static void OnMouseMove(object sender, MouseEventArgs e)
        //{
        //    UIElement element = sender as UIElement;
        //    if (ElementsBeingDragged.ContainsKey(element))
        //    {
        //        ZObjectDragInformation info = ElementsBeingDragged[element];
        //        Point currentPoint = e.GetPosition(null);

        //        double scale = info.getScaleFactor();

        //        Vector diff = Point.Subtract(currentPoint, info.OriginalPositionOnWindow);

        //        //double xdiff = (currentPoint.X - info.OriginalMousePositionOnWindow.X);
        //        //double ydiff = (currentPoint.Y - info.OriginalMousePositionOnWindow.Y);

        //        if (diff.Length > 5)
        //        {
        //            //if (!ZObjectDragDropBehavior.GetIsDragging(element))
        //            //{

        //            //    ZObjectDragDropBehavior.SetIsDragging(element, true);

        //            //Raise OnDragStartedEvent and set IsDragging to true
        //            //ZObjectDragDropEventArgs args = new ZObjectDragDropEventArgs();
        //            //args.RoutedEvent = ZObjectDragDropBehavior.DragStartedEvent;
        //            //args.Source = element;
        //            //args.CurrentDragPoint = e.GetPosition(null);
        //            //element.RaiseEvent(args);
        //            //element.SetValue(ZObjectDragDropBehavior.IsDraggingProperty, true);
        //            //}
        //            //else
        //            //{


        //            SnapBehavior.RaiseSnapFinishedEvent(element as ZComponent);
        //            //}

        //            ElementsBeingDragged.Remove(sender as UIElement);

        //            element.ReleaseMouseCapture();
        //            e.Handled = true;

        //            //Raise DragDeltaEvent
        //            ZObjectDragDropEventArgs args = new ZObjectDragDropEventArgs();
        //            args.CurrentDragPoint = currentPoint;
        //            args.RoutedEvent = ZObjectDragDropBehavior.DragDeltaEvent;
        //            args.Source = element;
        //            element.RaiseEvent(args);

        //            SetIsDragging(element, false);
        //        }

        //        //double newX = info.Left + diff.X / scale;
        //        //double newY = info.Top + diff.Y / scale;

        //        //Canvas.SetLeft(info.ElementBeingDragged, newX);
        //        //Canvas.SetTop(info.ElementBeingDragged, newY);

        //        //e.Handled = true;
        //    }
        //}

        #endregion

        



        public static void SetIsDraggable(ZComponent target, bool value)
        {
            target.SetValue(ZObjectDragDropBehavior.IsDraggableProperty, value);
        }

        public static bool GetIsDraggable(ZComponent d)
        {
            return (bool)d.GetValue(ZObjectDragDropBehavior.IsDraggableProperty);
        }



        #endregion

        #region OnDragStarted attached property (for setting the RoutedCommand which is executed on UIElement after drag is started)

        public static DependencyProperty OnDragStartedProperty = DependencyProperty.RegisterAttached("OnDragStarted",
                    typeof(RoutedCommand),
                    typeof(ZObjectDragDropBehavior),
                    new FrameworkPropertyMetadata(null, new PropertyChangedCallback(ZObjectDragDropBehavior.OnDragStartedChanged)));

        private static void OnDragStartedChanged(DependencyObject target, DependencyPropertyChangedEventArgs e)
        {
            ZBehavior.HandleChange(e.OldValue, e.NewValue, target as UIElement, ZObjectDragDropBehavior.DragStartedEvent, new DragDropEventHandler(OnDragStarted));
        }

        static void OnDragStarted(object sender, ZObjectDragDropEventArgs args)
        {
            ZBehavior.ExecuteCommand(sender as UIElement, ZObjectDragDropBehavior.OnDragStartedProperty, args);
        }

        public static void SetOnDragStarted(DependencyObject target, RoutedCommand value)
        {
            target.SetValue(ZObjectDragDropBehavior.OnDragStartedProperty, value);
        }

        public static RoutedCommand GetOnDragStarted(DependencyObject d)
        {
            return d.GetValue(ZObjectDragDropBehavior.OnDragStartedProperty) as RoutedCommand;
        }

        #endregion

        #region OnDragDelta attached property (for setting the RoutedCommand which is executed on UIElement while dragging position changes)

        public static DependencyProperty OnDragDeltaProperty = DependencyProperty.RegisterAttached("OnDragDelta",
                    typeof(RoutedCommand),
                    typeof(ZObjectDragDropBehavior),
                    new FrameworkPropertyMetadata(null, new PropertyChangedCallback(ZObjectDragDropBehavior.OnDragDeltaChanged)));

        private static void OnDragDeltaChanged(DependencyObject target, DependencyPropertyChangedEventArgs e)
        {
            ZBehavior.HandleChange(e.OldValue, e.NewValue, target as UIElement, ZObjectDragDropBehavior.DragDeltaEvent, new DragDropEventHandler(OnDragDelta));
        }

        static void OnDragDelta(object sender, ZObjectDragDropEventArgs args)
        {
            ZBehavior.ExecuteCommand(sender as UIElement, ZObjectDragDropBehavior.OnDragDeltaProperty, args);
        }

        public static void SetOnDragDelta(DependencyObject target, RoutedCommand value)
        {
            target.SetValue(ZObjectDragDropBehavior.OnDragDeltaProperty, value);
        }

        public static RoutedCommand GetOnDragDelta(DependencyObject d)
        {
            return d.GetValue(ZObjectDragDropBehavior.OnDragDeltaProperty) as RoutedCommand;
        }

        #endregion

        #region OnDragFinished attached property (for setting the RoutedCommand which is executed on UIElement while dragging is finished)

        public static DependencyProperty OnDragFinishedProperty = DependencyProperty.RegisterAttached("OnDragFinished",
                    typeof(RoutedCommand),
                    typeof(ZObjectDragDropBehavior),
                    new FrameworkPropertyMetadata(null, new PropertyChangedCallback(ZObjectDragDropBehavior.OnDragStartedFinished)));

        private static void OnDragStartedFinished(DependencyObject target, DependencyPropertyChangedEventArgs e)
        {
            ZBehavior.HandleChange(e.OldValue, e.NewValue, target as UIElement, ZObjectDragDropBehavior.DragFinishedEvent, new DragDropEventHandler(OnDragFinished));
        }

        static void OnDragFinished(object sender, ZObjectDragDropEventArgs args)
        {
            ZBehavior.ExecuteCommand(sender as UIElement, ZObjectDragDropBehavior.OnDragFinishedProperty, args);
        }

        public static void SetOnDragFinished(DependencyObject target, RoutedCommand value)
        {
            target.SetValue(ZObjectDragDropBehavior.OnDragFinishedProperty, value);
        }

        public static RoutedCommand GetOnDragFinished(DependencyObject d)
        {
            return d.GetValue(ZObjectDragDropBehavior.OnDragFinishedProperty) as RoutedCommand;
        }

        #endregion

        #region DropBehaviors attacted property (for registering commands, that are executed when a specific object is dropped onto another

        public static ZObjectDropBehaviorCollection GetDropBehaviors(DependencyObject obj)
        {
            return (ZObjectDropBehaviorCollection)obj.GetValue(DropBehaviorsProperty);
        }

        public static void SetDropBehaviors(DependencyObject obj, ZObjectDropBehaviorCollection value)
        {
            obj.SetValue(DropBehaviorsProperty, value);
        }

        // Using a DependencyProperty as the backing store for DropBehaviors.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty DropBehaviorsProperty =
            DependencyProperty.RegisterAttached("DropBehaviors", typeof(ZObjectDropBehaviorCollection), typeof(ZObjectDragDropBehavior), new UIPropertyMetadata(null, OnDropBehaviorsChanged));

        private static void OnDropBehaviorsChanged(DependencyObject sender, DependencyPropertyChangedEventArgs e)
        {
            if (e.OldValue == null)
            {
                foreach (ZObjectDropBehavior behavior in (e.NewValue as ZObjectDropBehaviorCollection))
                {
                    RegisterDropBehavior(sender, behavior);
                }
            }
        }

        private static Dictionary<UIElement, ZObjectDropBehavior> _dropRegistry = new Dictionary<UIElement, ZObjectDropBehavior>();

        private static void RegisterDropBehavior(DependencyObject sender, ZObjectDropBehavior behavior)
        {
            _dropRegistry.Add(sender as UIElement, behavior);
        }

        #endregion

        #endregion
    }

    public class ZObjectDragDropEventArgs : RoutedEventArgs
    {
        public ZObjectDragDropEventArgs() : base()
        {

        }

        public Point CurrentDragPoint
        {
            get;
            set;
        }
    }
}
