﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Input;
using Microsoft.Surface.Presentation;
using ZOIL.Framework.Presentation;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Media;
using System.Windows.Controls.Primitives;
using Microsoft.Surface.Presentation.Manipulations;
using ZOIL.Framework.Core;

namespace ZOIL.Library.Behaviors
{
    public static class ZObjectResizeBehavior
    {

        #region IsResizable attached property (for setting resizing behavior active or unactive)

        public static DependencyProperty IsResizableProperty = DependencyProperty.RegisterAttached("IsResizable",
                    typeof(bool),
                    typeof(ZObjectResizeBehavior),
                    new FrameworkPropertyMetadata(false, new PropertyChangedCallback(ZObjectResizeBehavior.IsResizableChanged)));

        private static void IsResizableChanged(DependencyObject target, DependencyPropertyChangedEventArgs e)
        {
            UIElement element = target as UIElement;

            /* MZ: Commented out due to performance issues
            ZBehavior.HandleChange(e.OldValue, e.NewValue, target as UIElement, Mouse.MouseEnterEvent, new MouseEventHandler(OnMouseEnter));
            ZBehavior.HandleChange(e.OldValue, e.NewValue, target as UIElement, Mouse.MouseLeaveEvent, new MouseEventHandler(OnMouseLeave));
            */
            //Surface



            ZBehavior.HandleChange(e.OldValue, e.NewValue, target as UIElement, Contacts.ContactDownEvent, new ContactEventHandler(ZObjectSurfaceHandler.OnContactDown));
            ZBehavior.HandleChange(e.OldValue, e.NewValue, target as UIElement, Contacts.ContactUpEvent, new ContactEventHandler(ZObjectSurfaceHandler.OnContactUp));
            ZBehavior.HandleChange(e.OldValue, e.NewValue, target as UIElement, Contacts.ContactTapGestureEvent, new ContactEventHandler(ZObjectSurfaceHandler.OnContactTapGesture));
            //ZBehavior.HandleChange(e.OldValue, e.NewValue, target as UIElement, Contacts.ContactChangedEvent, new ContactEventHandler(OnContactChanged));
            ZBehavior.HandleChange(e.OldValue, e.NewValue, target as UIElement, Contacts.ContactHoldGestureEvent, new ContactEventHandler(ZObjectSurfaceHandler.OnContactHold));

            //TODO: enable these handlers, so the event handling with processors is cleaner
            //ZBehavior.HandleChange(e.OldValue, e.NewValue, target as UIElement, Contacts.ContactUpEvent, new ContactEventHandler(ZObjectSurfaceHandler.OnContactUp));
            //ZBehavior.HandleChange(e.OldValue, e.NewValue, target as UIElement, Contacts.GotContactCaptureEvent, new ContactEventHandler(ZObjectSurfaceHandler.OnGotContactCapture));
            //ZBehavior.HandleChange(e.OldValue, e.NewValue, target as UIElement, Contacts.LostContactCaptureEvent, new ContactEventHandler(ZObjectSurfaceHandler.OnLostContactCapture));


            //Wii

        }

        private static Dictionary<FrameworkElement, ZObjectResizingInformation> ElementsBeingResized = new Dictionary<FrameworkElement, ZObjectResizingInformation>();


        #region mouse handling

        static Adorner currentAdorner;
        static AdornerLayer currentAdornerLayer;

        private static void OnMouseEnter(object sender, MouseEventArgs e)
        {
            ZComponent element = sender as ZComponent;

            if (element != null)
            {
                AdornerLayer layer = AdornerLayer.GetAdornerLayer(element);
                if (layer != null)
                {
                    ZObjectResizeAdorner resizeAdorner = new ZObjectResizeAdorner(element);
                    layer.Add(resizeAdorner);
                }
            }
        }

        private static void OnMouseLeave(object sender, MouseEventArgs e)
        {
            ZComponent element = sender as ZComponent;
            if (element != null)
            {
                AdornerLayer layer = AdornerLayer.GetAdornerLayer(element);
                if (layer != null)
                {
                    currentAdornerLayer = layer;
                    var resizeAdorners = layer.GetAdorners(element).OfType<ZObjectResizeAdorner>();
                    foreach (ZObjectResizeAdorner resizeAdorner in resizeAdorners)
                    {
                        if (resizeAdorner.AdornedElement == element)
                        {
                            if (resizeAdorner.IsMouseOver)
                            {
                                currentAdorner = resizeAdorner;
                                resizeAdorner.MouseLeave += new MouseEventHandler(resizeAdorner_MouseLeave);
                                return;
                            }
                            layer.Remove(resizeAdorner);
                        }
                    }
                }
            }
        }

        static void resizeAdorner_MouseLeave(object sender, MouseEventArgs e)
        {
            currentAdornerLayer.Remove(currentAdorner);
        }

        #endregion

        //#region surface handling

        //private static void OnContactDown(object sender, ContactEventArgs e)
        //{
        //    FrameworkElement element = sender as FrameworkElement;
        //    if (element != null)
        //    {
        //        ZObjectResizingInformation info = null;
        //        if (!ElementsBeingResized.ContainsKey(element))
        //        {
        //            info = new ZObjectResizingInformation(element);
        //            ElementsBeingResized[element] = info;
        //            info.ManipulationProcessor.Affine2DManipulationDelta += new EventHandler<Affine2DOperationDeltaEventArgs>(ManipulationProcessor_Affine2DManipulationDelta);
        //            info.ManipulationProcessor.Affine2DManipulationCompleted += new EventHandler<Affine2DOperationCompletedEventArgs>(ManipulationProcessor_Affine2DManipulationCompleted);
        //        }
        //        else
        //        {
        //            info = ElementsBeingResized[element];
        //        }
        //        info.ContactEvents.Add(e);
        //        info.ManipulationProcessor.BeginTrack(e.Contact);
        //        e.Handled = true;
        //    }
        //}

        //static void ManipulationProcessor_Affine2DManipulationCompleted(object sender, Affine2DOperationCompletedEventArgs e)
        //{
        //    Affine2DManipulationProcessor processor = sender as Affine2DManipulationProcessor;

        //    FrameworkElement fe = processor.Container as FrameworkElement;
        //    if (fe != null)
        //    {
        //        if (ElementsBeingResized.ContainsKey(fe))
        //        {
        //            ElementsBeingResized.Remove(fe);
        //        }
        //    }
        //}

        //static void ManipulationProcessor_Affine2DManipulationDelta(object sender, Affine2DOperationDeltaEventArgs e)
        //{
            

        //    Affine2DManipulationProcessor processor = sender as Affine2DManipulationProcessor;

        //    FrameworkElement fe = processor.Container as FrameworkElement;
        //    if (fe != null)
        //    {

        //        if (ElementsBeingResized.ContainsKey(fe))
        //        {
        //            ZObjectResizingInformation info = ElementsBeingResized[fe];

        //            Console.WriteLine(e.ScaleDelta);

                    
        //            double oldWidth = info.ElementBeingResized.Width;
        //            double oldHeight = info.ElementBeingResized.Height;

        //            info.ElementBeingResized.Width *= e.ScaleDelta;
        //            info.ElementBeingResized.Height *= e.ScaleDelta;

        //            double xdiff = info.ElementBeingResized.Width - oldWidth;
        //            double ydiff = info.ElementBeingResized.Height - oldHeight;

        //            Canvas.SetLeft(info.ElementBeingResized, Canvas.GetLeft(info.ElementBeingResized) - (xdiff / 2.0));
        //            Canvas.SetTop(info.ElementBeingResized, Canvas.GetTop(info.ElementBeingResized) - (ydiff / 2.0));
                    

        //            foreach (ContactEventArgs args in info.ContactEvents)
        //            {
        //                args.Handled = true;
                        
        //            }

        //        }
        //    }
        //}

        //private static void OnContactChanged(object sender, ContactEventArgs e)
        //{
        //    FrameworkElement element = sender as FrameworkElement;
        //    if (element != null)
        //    {
        //        if(ElementsBeingResized.ContainsKey(element))
        //        {
        //            ZObjectResizingInformation info = ElementsBeingResized[element];
        //            info.ContactEvents.Add(e);
        //        }
        //    }
        //}

        //private static void OnContactLeave(object sender, ContactEventArgs e)
        //{

        //}

        //#endregion

        public static void SetIsResizable(UIElement target, bool value)
        {
            target.SetValue(ZObjectResizeBehavior.IsResizableProperty, value);
        }

        public static bool GetIsResizable(UIElement d)
        {
            return (bool)d.GetValue(ZObjectResizeBehavior.IsResizableProperty);
        }
        #endregion

    }

    public class ZObjectResizingInformation
    {
        public ZObjectResizingInformation(FrameworkElement element)
        {
            ElementBeingResized = element;
            ManipulationProcessor = new Affine2DManipulationProcessor(Affine2DManipulations.Scale | Affine2DManipulations.TranslateX | Affine2DManipulations.TranslateY, ElementBeingResized, true);
            ContactEvents = new List<ContactEventArgs>();
        }

        public Affine2DManipulationProcessor ManipulationProcessor;

        public List<ContactEventArgs> ContactEvents;

        public FrameworkElement ElementBeingResized;
        public List<ZInformationLandscape> ParentLandscapes;
        
        public double getScaleFactor()
        {
            if (ParentLandscapes == null)
            {
                return 1.0;
            }
            else
            {
                double scale = 1.0;
                foreach (ZInformationLandscape landscape in ParentLandscapes)
                {
                    scale *= landscape.Animation.Position.Zoom;
                }
                return scale;
            }
        }
    }

    public class ZObjectResizeAdorner : ZComponentAdorner
    {
        private Thumb _leftThumb, _rightThumb, _bottomThumb, _bottomRightThumb, _bottomLeftThumb, _topLeftThumb, _topThumb, _topRightThumb;

        private double _scaleFactor = 0.0;

        private double _thumbSize = 10.0;

        private double _currentAdornedElementWidth;

        private double _currentAdornedElementHeight;

        private bool _isVisible = true;

        #region constructor

        public ZObjectResizeAdorner(ZComponent elem) : base(elem)
        {
            // find parent landscape to get current scalefactor
            ZTreeHelper.TryFindParent<ZInformationLandscape>(elem).Animation.Changed += Animation_Changed;

            _scaleFactor = ZTreeHelper.TryFindParent<ZInformationLandscape>(elem).Animation.Position.Zoom;

            #region calculate current AdornedElement-Size, that Thumbs are only visible if AdornerElement is large enough

            AdornerLayer layer = AdornerLayer.GetAdornerLayer(AdornedElement);
            _currentAdornedElementWidth = AdornedElement.DesiredSize.Width * _scaleFactor;
            _currentAdornedElementHeight = AdornedElement.DesiredSize.Height * _scaleFactor;

            if ((_currentAdornedElementWidth < _thumbSize * 4) || (_currentAdornedElementHeight < _thumbSize * 4))
            {
                //initThumbs();
                layer.Remove(this);
                _isVisible = false;
                return;
            }
            else
            {
                _isVisible = true;
                initThumbs();
            }

            #endregion
        }

        /// <summary>
        /// Initializes thumbs depending on current scalefactor
        /// </summary>
        private void initThumbs()
        {
            _rightThumb = getAdornerCorner(Cursors.SizeWE, _thumbSize / _scaleFactor, AdornedElement.DesiredSize.Height);
            _leftThumb = getAdornerCorner(Cursors.SizeWE, _thumbSize / _scaleFactor, AdornedElement.DesiredSize.Height);
            _bottomThumb = getAdornerCorner(Cursors.SizeNS, AdornedElement.DesiredSize.Width, _thumbSize / _scaleFactor);
            _topThumb = getAdornerCorner(Cursors.SizeNS, AdornedElement.DesiredSize.Width, _thumbSize / _scaleFactor);

            _bottomRightThumb = getAdornerCorner(Cursors.SizeNWSE, _thumbSize / _scaleFactor, _thumbSize / _scaleFactor);
            _bottomLeftThumb = getAdornerCorner(Cursors.SizeNESW, _thumbSize / _scaleFactor, _thumbSize / _scaleFactor);
            _topRightThumb = getAdornerCorner(Cursors.SizeNESW, _thumbSize / _scaleFactor, _thumbSize / _scaleFactor);
            _topLeftThumb = getAdornerCorner(Cursors.SizeNWSE, _thumbSize / _scaleFactor, _thumbSize / _scaleFactor);


            this.AddChild(_rightThumb, AdornerPlacement.In, AdornerPosition.RightEdge, 0, 0);
            this.AddChild(_leftThumb, AdornerPlacement.In, AdornerPosition.LeftEdge, 0, 0);

            this.AddChild(_bottomThumb, AdornerPlacement.In, AdornerPosition.BottomEdge, 0, 0);
            this.AddChild(_topThumb, AdornerPlacement.In, AdornerPosition.TopEdge, 0, 0);

            this.AddChild(_bottomRightThumb, AdornerPlacement.In, AdornerPosition.BottomRightcorner);
            this.AddChild(_bottomLeftThumb, AdornerPlacement.In, AdornerPosition.BottomLeftCorner);

            this.AddChild(_topRightThumb, AdornerPlacement.In, AdornerPosition.TopRightCorner);
            this.AddChild(_topLeftThumb, AdornerPlacement.In, AdornerPosition.TopLeftCorner);


            _rightThumb.DragDelta += new DragDeltaEventHandler(HandleRight);
            _bottomThumb.DragDelta += new DragDeltaEventHandler(HandleBottom);
            _bottomRightThumb.DragDelta += new DragDeltaEventHandler(HandleBottomRight);
            _leftThumb.DragDelta += new DragDeltaEventHandler(HandleLeft);
            _bottomLeftThumb.DragDelta += new DragDeltaEventHandler(HandleBottomLeft);
            _topLeftThumb.DragDelta += new DragDeltaEventHandler(HandleTopLeft);
            _topThumb.DragDelta += new DragDeltaEventHandler(HandleTop);
            _topRightThumb.DragDelta += new DragDeltaEventHandler(HandleTopRight);
        }

        /// <summary>
        /// if Landscape-Animations changes, size of thumbs get reset
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        public void Animation_Changed(object sender, ZLandscapeAnimationEventArgs e)
        {
            // get current scalefactor if Animation exists:
            ZInformationLandscape zil = ZTreeHelper.TryFindParent<ZInformationLandscape>(AdornedElement);

            if (zil != null)
            {
                _scaleFactor = zil.Animation.Position.Zoom;

                _currentAdornedElementWidth = AdornedElement.DesiredSize.Width * _scaleFactor;
                _currentAdornedElementHeight = AdornedElement.DesiredSize.Height * _scaleFactor;

                AdornerLayer layer = AdornerLayer.GetAdornerLayer(AdornedElement);

                // if AdornedElement is not large enough, remove adorner from AdornerLayer
                if (((_currentAdornedElementWidth < _thumbSize * 4) || (_currentAdornedElementHeight < _thumbSize * 4)))
                {
                    layer.Remove(this);
                    _isVisible = false;
                    return;
                }

                // if AdornedElement is now large enough and AdornerLayer does not contain adorner, add it.
                if (!_isVisible)
                {
                    if(!layer.GetAdorners(AdornedElement).Contains(this))
                        layer.Add(this);
                    _isVisible = true;
                }

                //HACK: if thumbs are not set, initialize them
                if (_topThumb == null)
                    initThumbs();

                #region set size of thumbs depending on current scalefactor

                _topLeftThumb.Width = _thumbSize / _scaleFactor;
                _rightThumb.Width = _thumbSize / _scaleFactor;
                _leftThumb.Width = _thumbSize / _scaleFactor;
                _bottomRightThumb.Width = _thumbSize / _scaleFactor;
                _bottomLeftThumb.Width = _thumbSize / _scaleFactor;
                _topLeftThumb.Width = _thumbSize / _scaleFactor;
                _topRightThumb.Width = _thumbSize / _scaleFactor;

                _topLeftThumb.Height = _thumbSize / _scaleFactor;
                _bottomThumb.Height = _thumbSize / _scaleFactor;
                _bottomRightThumb.Height = _thumbSize / _scaleFactor;
                _bottomLeftThumb.Height = _thumbSize / _scaleFactor;
                _topLeftThumb.Height = _thumbSize / _scaleFactor;
                _topThumb.Height = _thumbSize / _scaleFactor;
                _topRightThumb.Height = _thumbSize / _scaleFactor;

                _rightThumb.Height = AdornedElement.DesiredSize.Height;
                _leftThumb.Height = AdornedElement.DesiredSize.Height;

                _bottomThumb.Width = AdornedElement.DesiredSize.Width;
                _topThumb.Width = AdornedElement.DesiredSize.Width;

                #endregion
            }
        }

        #endregion

        #region Visual Property Setters

        void AddToWidth(Thumb thumb, double diff)
        {
            FrameworkElement adornedElement = AdornedElement as FrameworkElement;
            if (adornedElement == null)
            {
                return;
            }
            adornedElement.Width = Math.Max(adornedElement.Width + diff, thumb.DesiredSize.Width);
        }

        void AddToHeight(Thumb thumb, double diff)
        {
            FrameworkElement adornedElement = AdornedElement as FrameworkElement;
            if (adornedElement == null)
            {
                return;
            }
            adornedElement.Height = Math.Max(adornedElement.Height + diff, thumb.DesiredSize.Height);
        }
        
        void AddToLeft(double diff)
        {
            FrameworkElement adornedElement = AdornedElement as FrameworkElement;
            if (adornedElement == null)
            {
                return;
            }
            Canvas.SetLeft(adornedElement, Canvas.GetLeft(adornedElement) + diff);
        }

        void AddToTop(double diff)
        {
            FrameworkElement adornedElement = AdornedElement as FrameworkElement;
            if (adornedElement == null)
            {
                return;
            }
            Canvas.SetTop(adornedElement, Canvas.GetTop(adornedElement) + diff);
        }

        #endregion

        #region Event Handling

        void HandleLeft(object sender, DragDeltaEventArgs args)
        {
            Thumb hitThumb = sender as Thumb;
            
            if (hitThumb == null) return;

            EnforceSize();
            
            AddToWidth(hitThumb, -args.HorizontalChange);
            AddToLeft(args.HorizontalChange);
        }

        void HandleBottomLeft(object sender, DragDeltaEventArgs args)
        {
            Thumb hitThumb = sender as Thumb;

            if (hitThumb == null) return;
            
            EnforceSize();
            
            AddToWidth(hitThumb, -args.HorizontalChange);
            AddToHeight(hitThumb, args.VerticalChange);
            AddToLeft(args.HorizontalChange);
        }

        void HandleTopLeft(object sender, DragDeltaEventArgs args)
        {
            Thumb hitThumb = sender as Thumb;

            if (hitThumb == null) return;
            
            EnforceSize();

            AddToWidth(hitThumb, -args.HorizontalChange);
            AddToHeight(hitThumb, -args.VerticalChange);
            AddToLeft(args.HorizontalChange);
            AddToTop(args.VerticalChange);
        }
        
        void HandleTop(object sender, DragDeltaEventArgs args)
        {
            Thumb hitThumb = sender as Thumb;

            if (hitThumb == null) return;
            
            EnforceSize();

            AddToHeight(hitThumb, -args.VerticalChange);
            AddToTop(args.VerticalChange);
        }
        
        void HandleTopRight(object sender, DragDeltaEventArgs args)
        {
            Thumb hitThumb = sender as Thumb;

            if (hitThumb == null) return;
            
            EnforceSize();

            AddToWidth(hitThumb, args.HorizontalChange);
            AddToHeight(hitThumb, -args.VerticalChange);
            AddToTop(args.VerticalChange);
        }
        
        void HandleRight(object sender, DragDeltaEventArgs args)
        {
            Thumb hitThumb = sender as Thumb;

            if (hitThumb == null) return;
            
            EnforceSize();

            AddToWidth(hitThumb, args.HorizontalChange);
        }

        void HandleBottom(object sender, DragDeltaEventArgs args)
        {
            Thumb hitThumb = sender as Thumb;

            if (hitThumb == null) return;

            EnforceSize();

            AddToHeight(hitThumb, args.VerticalChange);
        }

        void HandleBottomRight(object sender, DragDeltaEventArgs args)
        {
            Thumb hitThumb = sender as Thumb;

            if (hitThumb == null) return;

            EnforceSize();

            AddToWidth(hitThumb, args.HorizontalChange);
            AddToHeight(hitThumb, args.VerticalChange);
        }

        #endregion

        #region private helper methods

        private Thumb getAdornerCorner(Cursor customizedCursor, double w, double h)
        {
            Thumb cornerThumb = new Thumb();
            cornerThumb.Cursor = customizedCursor;
            cornerThumb.Width = (w < 0 ? 0 : w);
            cornerThumb.Height = (h < 0 ? 0 : h);
            cornerThumb.Opacity = 0;

            return cornerThumb;
        }

        private void EnforceSize()
        {
            FrameworkElement adornedElement = AdornedElement as FrameworkElement;

            if (adornedElement == null) return;

            if (adornedElement.Width.Equals(Double.NaN))
                adornedElement.Width = adornedElement.DesiredSize.Width;
            if (adornedElement.Height.Equals(Double.NaN))
                adornedElement.Height = adornedElement.DesiredSize.Height;

            FrameworkElement parent = adornedElement.Parent as FrameworkElement;
            if (parent != null)
            {
                adornedElement.MaxHeight = parent.ActualHeight;
                adornedElement.MaxWidth = parent.ActualWidth;
            }
        }
        
        #endregion

        #region Adorner overrides

        ////TODO: check if this method is necessary 
        //protected override Size ArrangeOverride(Size finalSize)
        //{
        //    double desiredWidth = Math.Max(AdornedElement.DesiredSize.Width, 20.0);
        //    double desiredHeight = Math.Max(AdornedElement.DesiredSize.Height, 20.0);
        //    double adornerWidth = this.DesiredSize.Width;
        //    double adornerHeight = this.DesiredSize.Height;

        //    base.ArrangeOverride(finalSize);
            

        //    _rightThumb.Arrange(new Rect(desiredWidth - 10, 10, 10, desiredHeight - 20));
        //    _bottomThumb.Arrange(new Rect(10, desiredHeight - 10, desiredWidth - 20, 10));
        //    _bottomRightThumb.Arrange(new Rect(desiredWidth - 10, desiredHeight - 10, 10, 10));
        //    _leftThumb.Arrange(new Rect(0, 10, 10, desiredHeight - 20));
        //    _bottomLeftThumb.Arrange(new Rect(0, desiredHeight - 10, 10, 10));
        //    _topLeftThumb.Arrange(new Rect(0, 0, 10, 10));
        //    _topThumb.Arrange(new Rect(10, 0, desiredWidth - 20, 10));
        //    _topRightThumb.Arrange(new Rect(desiredWidth - 10, 0, 10, 10));


        //    return finalSize;
        //}
        
        //protected override int VisualChildrenCount
        //{
        //    get
        //    {
        //        return visualChildren.Count;
        //    }
        //}

        //protected override Visual GetVisualChild(int index)
        //{
        //    return visualChildren[index];
        //}

        #endregion
    }
}
