﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Input;
using Microsoft.Surface.Presentation;
using System.Windows.Controls;
using ZOIL.Framework.Presentation;
using System.Windows.Controls.Primitives;
using System.Windows.Documents;
using System.Windows.Media;

namespace ZOIL.Library.Behaviors
{
    public static class ZObjectRotateBehavior
    {
        #region IsRotatable attached property (for setting rotation behavior active or unactive)

        public static DependencyProperty IsRotatableProperty = DependencyProperty.RegisterAttached("IsRotatable",
                    typeof(bool),
                    typeof(ZObjectRotateBehavior),
                    new FrameworkPropertyMetadata(false, new PropertyChangedCallback(ZObjectRotateBehavior.IsRotatableChanged)));

        private static void IsRotatableChanged(DependencyObject target, DependencyPropertyChangedEventArgs e)
        {
            UIElement element = target as UIElement;
            
            /* MZ: Temporarily commented out due to performance issues
            ZBehavior.HandleChange(e.OldValue, e.NewValue, target as UIElement, Mouse.MouseEnterEvent, new MouseEventHandler(OnMouseEnter));
            ZBehavior.HandleChange(e.OldValue, e.NewValue, target as UIElement, Mouse.MouseLeaveEvent, new MouseEventHandler(OnMouseLeave));
            */
            //Surface
            ZBehavior.HandleChange(e.OldValue, e.NewValue, target as UIElement, Contacts.ContactDownEvent, new ContactEventHandler(ZObjectSurfaceHandler.OnContactDown));
            ZBehavior.HandleChange(e.OldValue, e.NewValue, target as UIElement, Contacts.ContactUpEvent, new ContactEventHandler(ZObjectSurfaceHandler.OnContactUp));
            ZBehavior.HandleChange(e.OldValue, e.NewValue, target as UIElement, Contacts.ContactTapGestureEvent, new ContactEventHandler(ZObjectSurfaceHandler.OnContactTapGesture));
            //ZBehavior.HandleChange(e.OldValue, e.NewValue, target as UIElement, Contacts.ContactChangedEvent, new ContactEventHandler(OnContactChanged));
            //ZBehavior.HandleChange(e.OldValue, e.NewValue, target as UIElement, Contacts.ContactLeaveEvent, new ContactEventHandler(OnContactLeave));
            ZBehavior.HandleChange(e.OldValue, e.NewValue, target as UIElement, Contacts.ContactHoldGestureEvent, new ContactEventHandler(ZObjectSurfaceHandler.OnContactHold));

            //Wii
        }

        private static Dictionary<UIElement, ZObjectRotationInformation> ElementsBeingRotated = new Dictionary<UIElement, ZObjectRotationInformation>();

        #region mouse handling

        static Adorner currentAdorner;
        static AdornerLayer currentAdornerLayer;

        private static void OnMouseEnter(object sender, MouseEventArgs e)
        {
            ZComponent element = sender as ZComponent;

            if (element != null)
            {
                AdornerLayer layer = AdornerLayer.GetAdornerLayer(element);
                if (layer != null)
                {
                    ZObjectRotateAdorner resizeAdorner = new ZObjectRotateAdorner(element);
                    layer.Add(resizeAdorner);
                }
            }
        }

        private static void OnMouseLeave(object sender, MouseEventArgs e)
        {
            UIElement element = sender as UIElement;
            ZComponent comp = element as ZComponent;
            if (comp != null)
            {
                AdornerLayer layer = AdornerLayer.GetAdornerLayer(comp);
                if (layer != null)
                {
                    currentAdornerLayer = layer;
                    var rotateAdorners = layer.GetAdorners(comp).OfType<ZObjectRotateAdorner>();
                    foreach (ZObjectRotateAdorner rotateAdorner in rotateAdorners)
                    {
                        if (rotateAdorner.AdornedElement == comp)
                        {
                            Rect test = new Rect(e.GetPosition(null).X, e.GetPosition(null).Y, comp.Width, comp.Height);
                            if (test.Contains(e.GetPosition(null)))
                            {
                                currentAdorner = rotateAdorner;
                                rotateAdorner.MouseLeave += new MouseEventHandler(rotateAdorner_MouseLeave);
                                return;
                            }
                            layer.Remove(rotateAdorner);
                        }
                    }
                }
            }
        }

        static void rotateAdorner_MouseLeave(object sender, MouseEventArgs e)
        {
            currentAdornerLayer.Remove(currentAdorner);
        }

        #endregion

        #region surface handling

        private static void OnContactDown(object sender, ContactEventArgs e)
        {

        }

        private static void OnContactChanged(object sender, ContactEventArgs e)
        {

        }

        private static void OnContactLeave(object sender, ContactEventArgs e)
        {

        }

        #endregion

        public static void SetIsRotatable(UIElement target, bool value)
        {
            target.SetValue(ZObjectRotateBehavior.IsRotatableProperty, value);
        }

        public static bool GetIsRotatable(UIElement d)
        {
            return (bool)d.GetValue(ZObjectRotateBehavior.IsRotatableProperty);
        }
        
        #endregion
    }

    public class ZObjectRotationInformation
    {
        public ZObjectRotationInformation(Point originalMousePosition)
        {
            OriginalMousePositionOnWindow = originalMousePosition;
        }

        public Point OriginalMousePositionOnWindow;
        public Point OriginalMousePositionOnUIElement;
        public double Top;
        public double Bottom;
        public double Left;
        public double Right;
        public UIElement ElementBeingDragged;
        public List<ZInformationLandscape> ParentLandscapes;
        public bool IsDragging;

        public double getScaleFactor()
        {
            if (ParentLandscapes == null)
            {
                return 1.0;
            }
            else
            {
                double scale = 1.0;
                foreach (ZInformationLandscape landscape in ParentLandscapes)
                {
                    scale *= landscape.Animation.Position.Zoom;
                }
                return scale;
            }
        }
    }

    public class ZObjectRotateAdorner : ZComponentAdorner
    {
        Thumb _topLeft;
        Thumb _topRight;
        Thumb _bottomLeft;
        Thumb _bottomRight;

        private Vector _startVector;
        private Point _centerPoint;

        public ZObjectRotateAdorner(ZComponent adornedElement) : base(adornedElement)
        {
            //TODO: this is a mean hack to avoid adorners
            return;
            
            _topLeft = getAdornerCorner(Cursors.Cross, 10, 10);
            _topRight = getAdornerCorner(Cursors.Cross, 10, 10);
            _bottomLeft = getAdornerCorner(Cursors.Cross, 10, 10);
            _bottomRight = getAdornerCorner(Cursors.Cross, 10, 10);
            
            AddChild(_topLeft, AdornerPlacement.Out, AdornerPosition.TopLeftCorner);
            AddChild(_topRight, AdornerPlacement.CenterOnEdge, AdornerPosition.TopRightCorner);
            AddChild(_bottomLeft, AdornerPlacement.CenterOnEdge, AdornerPosition.BottomLeftCorner);
            AddChild(_bottomRight, AdornerPlacement.CenterOnEdge, AdornerPosition.BottomRightcorner);

            _topLeft.DragStarted += new DragStartedEventHandler(OnDragStarted);
            _topLeft.DragDelta += new DragDeltaEventHandler(OnDragDelta);
            _topRight.DragStarted += new DragStartedEventHandler(OnDragStarted);
            _topRight.DragDelta += new DragDeltaEventHandler(OnDragDelta);
            _bottomLeft.DragStarted += new DragStartedEventHandler(OnDragStarted);
            _bottomLeft.DragDelta += new DragDeltaEventHandler(OnDragDelta);
            _bottomRight.DragStarted += new DragStartedEventHandler(OnDragStarted);
            _bottomRight.DragDelta += new DragDeltaEventHandler(OnDragDelta);
        }

        void OnDragDelta(object sender, DragDeltaEventArgs e)
        {
            Vector currentVector = Point.Subtract(_centerPoint, Mouse.GetPosition(null));
            double angle = Vector.AngleBetween(_startVector, currentVector);
            AdornedComponent.Angle += angle;
        }

        void OnDragStarted(object sender, DragStartedEventArgs e)
        {
            Point centerPointInternal = new Point(AdornedComponent.Width / 2.0, AdornedComponent.Height / 2.0);
            _centerPoint = (sender as UIElement).TranslatePoint(centerPointInternal, Window.GetWindow(sender as UIElement));
            _startVector = Point.Subtract(_centerPoint, Mouse.GetPosition(null));
        }

        private Thumb getAdornerCorner(Cursor customizedCursor, double w, double h)
        {
            Thumb cornerThumb = new Thumb();
            cornerThumb.Cursor = customizedCursor;
            cornerThumb.Width = (w < 0 ? 0 : w);
            cornerThumb.Height = (h < 0 ? 0 : h);
            
            return cornerThumb;
        }

    }

}