﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Interactivity;
using System.Windows;
using System.ComponentModel;
using System.Windows.Data;

namespace ZOIL.Library.Behaviors
{
    public class PluginBehavior<T, P> : PluginBehavior<T>, INotifyPropertyChanged where P:Behavior<T> where T:DependencyObject
    {
        #region ParentBehavior

        public new P ParentBehavior
        {
            get 
            { 
                return (P)base.ParentBehavior; 
            }
            set
            {
                base.ParentBehavior = value;
            }
        }

        /// <summary>
        /// Provides derived classes an opportunity to handle changes to the ParentBehavior property.
        /// </summary>
        protected virtual void OnParentBehaviorChanged(P oldParentBehavior, P newParentBehavior)
        {

        }

        protected override void OnParentBehaviorChanged(object oldParentBehavior, object newParentBehavior)
        {
            this.OnParentBehaviorChanged((P)oldParentBehavior, (P)newParentBehavior);
        }

        #endregion

        
        protected void CreateOneWayParentBehaviorBinding(DependencyProperty parentProperty, DependencyProperty pluginProperty)
        {
            Binding b = new Binding();
            b.Source = ParentBehavior;
            b.Path = new PropertyPath(parentProperty);
            b.Mode = BindingMode.OneWay;
            
            BindingOperations.SetBinding(this, pluginProperty, b);
        }
    }

    public class PluginBehavior<T> : Behavior<T>, INotifyPropertyChanged where T : DependencyObject
    {

        #region ParentBehavior

        private object _parentBehavior;

        /// <summary>
        /// Gets or sets the ParentBehavior property. This observable property 
        /// indicates
        /// </summary>
        public object ParentBehavior
        {
            get { return _parentBehavior; }
            set
            {
                if (_parentBehavior != value)
                {
                    object old = _parentBehavior;
                    _parentBehavior = value;
                    OnParentBehaviorChanged(old, value);
                    RaisePropertyChanged("ParentBehavior");
                }
            }
        }

        /// <summary>
        /// Provides derived classes an opportunity to handle changes to the ParentBehavior property.
        /// </summary>
        protected virtual void OnParentBehaviorChanged(object oldParentBehavior, object newParentBehavior)
        {
        }

        #endregion

        #region INotifyPropertyChanged Members

        public event PropertyChangedEventHandler PropertyChanged;
        protected void RaisePropertyChanged(string name)
        {
            PropertyChangedEventHandler handler = PropertyChanged;
            if (handler != null) handler(this, new PropertyChangedEventArgs(name));
        }

        #endregion

    }
}
