﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Interactivity;
using System.ComponentModel.Composition;
using System.Windows;
using System.ComponentModel.Composition.Hosting;
using System.Reflection;

namespace ZOIL.Library.Behaviors
{
    public class RuntimeBehavior<T,B> : Behavior<T> where B:PluginBehavior<T> where T:DependencyObject
    {
        private static CompositionContainer _container;
        private static AggregateCatalog _catalog;
        private List<Tuple<RoutedEvent, Delegate>> _eventsToBeRegisteredOnAttaching;

        static RuntimeBehavior()
        {
            _catalog = new AggregateCatalog();
            _catalog.Catalogs.Add(new AssemblyCatalog(typeof(RuntimeBehavior<T, B>).Assembly));
            _container = new CompositionContainer(_catalog);
        }

        [ImportMany]
        protected IEnumerable<B> PluginBehaviors
        {
            get;
            set;
        }
  
        public RuntimeBehavior() : base()
        {

            Assembly currentAssembly = this.GetType().Assembly;

            if(!_catalog.Catalogs.OfType<AssemblyCatalog>().Any(catalog => catalog.Assembly == currentAssembly))
                _catalog.Catalogs.Add(new AssemblyCatalog(this.GetType().Assembly));

            //Fill the imports of this object
            try
            {
                _container.ComposeParts(this);
            }
            catch (CompositionException compositionException)
            {
                Console.WriteLine(compositionException.ToString());
            }

            foreach (B behavior in PluginBehaviors)
                behavior.ParentBehavior = this;

            _eventsToBeRegisteredOnAttaching = new List<Tuple<RoutedEvent, Delegate>>();

        }

        #region overrides

        protected override void OnAttached()
        {
            base.OnAttached();

            foreach (Tuple<RoutedEvent, Delegate> tuple in _eventsToBeRegisteredOnAttaching)
                RoutedEventHelper.AddHandler(AssociatedObject, tuple.Item1, tuple.Item2);

            foreach (B behavior in PluginBehaviors)
                behavior.Attach(AssociatedObject);
        }

        protected override void OnDetaching()
        {
            foreach (Tuple<RoutedEvent, Delegate> tuple in _eventsToBeRegisteredOnAttaching)
                RoutedEventHelper.RemoveHandler(AssociatedObject, tuple.Item1, tuple.Item2);

            foreach (B behavior in PluginBehaviors)
                behavior.Detach();

            base.OnDetaching();
        }

        #endregion

        protected void AddRoutedEventHandler(RoutedEvent evt, Delegate handler)
        {
            _eventsToBeRegisteredOnAttaching.Add(new Tuple<RoutedEvent, Delegate>(evt, handler));
        }

        protected void RemoveRoutedEventHandler(RoutedEvent evt, Delegate handler)
        {
            RoutedEventHelper.RemoveHandler(AssociatedObject, evt, handler);
        }
    }
}
