﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel.Composition;
using System.Windows.Input;

namespace ZOIL.Library.Behaviors
{
    [Export(typeof(SelectionPluginBehavior))]
    public class MouseSelectionBehavior : SelectionPluginBehavior
    {
        #region private fields

        private IDisposable _mouseSelectionSubscription;

        #endregion

        #region ctor

        public MouseSelectionBehavior()
            : base()
        {

        }

        #endregion

        #region private methods

        private IDisposable RegisterMouseEvents()
        {
            var mouseMove = Observable.FromEvent<MouseEventArgs>(AssociatedObject, "MouseMove");

            var mouseLeftButtonDown = Observable.FromEvent<MouseButtonEventArgs>(AssociatedObject, "MouseLeftButtonDown");

            var mouseLeftButtonUp = Observable.FromEvent<MouseButtonEventArgs>(AssociatedObject, "MouseLeftButtonUp");

            var mouseMoves = from mm in mouseMove
                             select mm.EventArgs.GetPosition(null);

            var mouseDrag = from mlbd in mouseLeftButtonDown.Select(evt => new Selection(evt.EventArgs.GetPosition(null))).Do(s => { StartSelection(s); Mouse.Capture(AssociatedObject); })
                            from md in mouseMoves.Do(point => mlbd.Points.Add(point)).TakeUntil(mouseLeftButtonUp.Do((evt) => mlbd.Points.Add(evt.EventArgs.GetPosition(null))).Do((evt) => { FinishSelection(mlbd); AssociatedObject.ReleaseMouseCapture(); }))
                            select mlbd;

            return mouseDrag.Subscribe(ChangeSelection);
        }

        #endregion

        #region overrides

        protected override void OnAttached()
        {
            base.OnAttached();
        }

        protected override void OnDetaching()
        {
            base.OnDetaching();
        }

        protected override void OnIsSelectionEnabledChanged(bool oldIsSelectionEnabled, bool newIsSelectionEnabled)
        {
            if (newIsSelectionEnabled)
            {
                _mouseSelectionSubscription = RegisterMouseEvents();
            }
            else
            {
                if (_mouseSelectionSubscription != null)
                    _mouseSelectionSubscription.Dispose();
            }
        }

        #endregion

        
    }
}
