﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Input;
using System.Windows.Shapes;
using System.Windows.Media;
using System.Windows.Controls;
using System.Windows.Media.Media3D;
using System.ComponentModel;
using ZOIL.Framework.Presentation;
using System.Windows.Interactivity;


namespace ZOIL.Library.Behaviors
{
    public delegate void SelectionEventHandler(object sender, SelectionEventArgs e);
    
    public class SelectionBehavior : RuntimeBehavior<FrameworkElement, SelectionPluginBehavior>
    {
        #region ctor

        public SelectionBehavior() : base()
        {

        }

        #endregion

        #region dp

        #region IsSelectionEnabled

        /// <summary>
        /// IsSelectionEnabled Dependency Property
        /// </summary>
        public static readonly DependencyProperty IsSelectionEnabledProperty =
            DependencyProperty.Register("IsSelectionEnabled", typeof(bool), typeof(SelectionBehavior),
                new FrameworkPropertyMetadata((bool)false, 
                    new PropertyChangedCallback(OnIsSelectionEnabledChanged)));

        /// <summary>
        /// Gets or sets the IsSelectionEnabled property. This dependency property 
        /// indicates ....
        /// </summary>
        public bool IsSelectionEnabled
        {
            get { return (bool)GetValue(IsSelectionEnabledProperty); }
            set { SetValue(IsSelectionEnabledProperty, value); }
        }

        /// <summary>
        /// Handles changes to the IsSelectionEnabled property.
        /// </summary>
        private static void OnIsSelectionEnabledChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            SelectionBehavior target = (SelectionBehavior)d;
            bool oldIsSelectionEnabled = (bool)e.OldValue;
            bool newIsSelectionEnabled = target.IsSelectionEnabled;
            target.OnIsSelectionEnabledChanged(oldIsSelectionEnabled, newIsSelectionEnabled);
        }

        /// <summary>
        /// Provides derived classes an opportunity to handle changes to the IsSelectionEnabled property.
        /// </summary>
        protected virtual void OnIsSelectionEnabledChanged(bool oldIsSelectionEnabled, bool newIsSelectionEnabled)
        {
            
        }

        #endregion

        #region SelectionMode

        /// <summary>
        /// SelectionMode Dependency Property
        /// </summary>
        public static readonly DependencyProperty SelectionModeProperty =
            DependencyProperty.Register("SelectionMode", typeof(SelectionModes), typeof(SelectionBehavior),
                new FrameworkPropertyMetadata((SelectionModes)SelectionModes.BoundingBox,
                    new PropertyChangedCallback(OnSelectionModeChanged)));

        /// <summary>
        /// Gets or sets the SelectionMode property. This dependency property 
        /// indicates ....
        /// </summary>
        public SelectionModes SelectionMode
        {
            get { return (SelectionModes)GetValue(SelectionModeProperty); }
            set { SetValue(SelectionModeProperty, value); }
        }

        /// <summary>
        /// Handles changes to the SelectionMode property.
        /// </summary>
        private static void OnSelectionModeChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            SelectionBehavior target = (SelectionBehavior)d;
            SelectionModes oldSelectionMode = (SelectionModes)e.OldValue;
            SelectionModes newSelectionMode = target.SelectionMode;
            target.OnSelectionModeChanged(oldSelectionMode, newSelectionMode);
        }

        /// <summary>
        /// Provides derived classes an opportunity to handle changes to the SelectionMode property.
        /// </summary>
        protected virtual void OnSelectionModeChanged(SelectionModes oldSelectionMode, SelectionModes newSelectionMode)
        {
        }

        #endregion

        #region TargetType

        /// <summary>
        /// TargetType Dependency Property
        /// </summary>
        public static readonly DependencyProperty TargetTypeProperty =
            DependencyProperty.Register("TargetType", typeof(Type), typeof(SelectionBehavior),
                new FrameworkPropertyMetadata((Type)typeof(DependencyObject),
                    new PropertyChangedCallback(OnTargetTypeChanged)));

        /// <summary>
        /// Gets or sets the TargetType property. This dependency property 
        /// indicates ....
        /// </summary>
        public Type TargetType
        {
            get { return (Type)GetValue(TargetTypeProperty); }
            set { SetValue(TargetTypeProperty, value); }
        }

        /// <summary>
        /// Handles changes to the TargetType property.
        /// </summary>
        private static void OnTargetTypeChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            SelectionBehavior target = (SelectionBehavior)d;
            Type oldTargetType = (Type)e.OldValue;
            Type newTargetType = target.TargetType;
            target.OnTargetTypeChanged(oldTargetType, newTargetType);
        }

        /// <summary>
        /// Provides derived classes an opportunity to handle changes to the TargetType property.
        /// </summary>
        protected virtual void OnTargetTypeChanged(Type oldTargetType, Type newTargetType)
        {
        }

        #endregion

        #endregion

        #region Routed Events

        #region Selection Finished

        public static readonly RoutedEvent SelectionFinishedEvent = EventManager.RegisterRoutedEvent("SelectionFinished", RoutingStrategy.Bubble, typeof(SelectionEventHandler), typeof(SelectionBehavior));

        public event SelectionEventHandler SelectionFinished
        {
            add { AddRoutedEventHandler(SelectionBehavior.SelectionFinishedEvent, value); }
            remove { RemoveRoutedEventHandler(SelectionBehavior.SelectionFinishedEvent, value); }
        }

        #endregion

        #region Selection Started

        public static readonly RoutedEvent SelectionStartedEvent = EventManager.RegisterRoutedEvent("SelectionStarted", RoutingStrategy.Bubble, typeof(SelectionEventHandler), typeof(SelectionBehavior));

        public event SelectionEventHandler SelectionStarted
        {
            add { AddRoutedEventHandler(SelectionBehavior.SelectionStartedEvent, value); }
            remove { RemoveRoutedEventHandler(SelectionBehavior.SelectionStartedEvent, value); }
        }

        #endregion

        #region Selection Changed

        public static readonly RoutedEvent SelectionChangedEvent = EventManager.RegisterRoutedEvent("SelectionChanged", RoutingStrategy.Bubble, typeof(SelectionChangedEventHandler), typeof(SelectionBehavior));

        public event SelectionChangedEventHandler SelectionChanged
        {
            add { AddRoutedEventHandler(SelectionBehavior.SelectionChangedEvent, value); }
            remove { RemoveRoutedEventHandler(SelectionBehavior.SelectionChangedEvent, value); }
        }

        #endregion

        #region Selection Cancelled

        public static readonly RoutedEvent SelectionCancelledEvent = EventManager.RegisterRoutedEvent("SelectionCancelled", RoutingStrategy.Bubble, typeof(SelectionEventHandler), typeof(SelectionBehavior));

        public event SelectionEventHandler SelectionCancelled
        {
            add { AddRoutedEventHandler(SelectionBehavior.SelectionCancelledEvent, value); }
            remove { RemoveRoutedEventHandler(SelectionBehavior.SelectionCancelledEvent, value); }
        }

        #endregion

        #endregion

        #region event helper

        internal void RaiseSelectionStartedEvent(Selection selection)
        {
            SelectionEventArgs args = new SelectionEventArgs(SelectionStartedEvent, selection);
            args.Source = this;
            AssociatedObject.RaiseEvent(args);
        }

        internal void RaiseSelectionChangedEvent(Selection selection)
        {
            SelectionEventArgs args = new SelectionEventArgs(SelectionChangedEvent, selection);
            args.Source = this;
            AssociatedObject.RaiseEvent(args);
        }

        internal void RaiseSelectionCancelledEvent(Selection selection)
        {
            SelectionEventArgs args = new SelectionEventArgs(SelectionCancelledEvent, selection);
            args.Source = this;
            AssociatedObject.RaiseEvent(args);
        }

        internal void RaiseSelectionFinishedEvent(Selection selection)
        {
            SelectionEventArgs args = new SelectionEventArgs(SelectionFinishedEvent, selection);
            args.Source = this;
            AssociatedObject.RaiseEvent(args);
        }

        #endregion
    }
}
