﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Documents;
using System.Windows.Media;
using ZOIL.Framework.Presentation;
using ZOIL.Framework.Core;

namespace ZOIL.Library.Behaviors
{
    public class SelectionPluginBehavior : PluginBehavior<FrameworkElement, SelectionBehavior>
    {
        #region private fields

        private AdornerLayer _adornerLayer;

        #endregion

        #region protected properties

        protected AdornerLayer AdornerLayer
        {
            get
            {
                if (_adornerLayer == null)
                    _adornerLayer = AdornerLayer.GetAdornerLayer(Window.GetWindow(AssociatedObject).Content as Visual);
                return _adornerLayer;
            }
        }

        #endregion

        #region ctor

        public SelectionPluginBehavior()
        {

        }

        #endregion
        
        #region overrides

        protected override void OnParentBehaviorChanged(SelectionBehavior oldParentBehavior, SelectionBehavior newParentBehavior)
        {
            CreateOneWayParentBehaviorBinding(SelectionBehavior.IsSelectionEnabledProperty, SelectionPluginBehavior.IsSelectionEnabledProperty);
            CreateOneWayParentBehaviorBinding(SelectionBehavior.SelectionModeProperty, SelectionPluginBehavior.SelectionModeProperty);
            CreateOneWayParentBehaviorBinding(SelectionBehavior.TargetTypeProperty, SelectionPluginBehavior.TargetTypeProperty);
        }

        protected override void OnAttached()
        {
            base.OnAttached();
        }

        #endregion

        #region dp

        #region IsSelectionEnabled

        /// <summary>
        /// Gets or sets the IsSelectionEnabled property. This dependency property 
        /// indicates ....
        /// </summary>
        public bool IsSelectionEnabled
        {
            get { return (bool)GetValue(IsSelectionEnabledProperty); }
            set { SetValue(IsSelectionEnabledProperty, value); }
        }

        public static readonly DependencyProperty IsSelectionEnabledProperty = SelectionBehavior.IsSelectionEnabledProperty.AddOwner(typeof(SelectionPluginBehavior), new FrameworkPropertyMetadata(false, OnIsSelectionEnabledChanged));

        private static void OnIsSelectionEnabledChanged(DependencyObject sender, DependencyPropertyChangedEventArgs e)
        {
            SelectionPluginBehavior target = (SelectionPluginBehavior)sender;
            bool oldIsSelectionEnabled = (bool)e.OldValue;
            bool newIsSelectionEnabled = target.IsSelectionEnabled;
            target.OnIsSelectionEnabledChanged(oldIsSelectionEnabled, newIsSelectionEnabled);
        }

        /// <summary>
        /// Provides derived classes an opportunity to handle changes to the IsSelectionEnabled property.
        /// </summary>
        protected virtual void OnIsSelectionEnabledChanged(bool oldIsSelectionEnabled, bool newIsSelectionEnabled)
        {

        }

        #endregion

        #region SelectionMode

        /// <summary>
        /// SelectionMode Dependency Property
        /// </summary>
        public static readonly DependencyProperty SelectionModeProperty = SelectionBehavior.SelectionModeProperty.AddOwner(typeof(SelectionPluginBehavior), new FrameworkPropertyMetadata(SelectionModes.BoundingBox, OnSelectionModeChanged));

        /// <summary>
        /// Gets or sets the SelectionMode property. This dependency property 
        /// indicates ....
        /// </summary>
        public SelectionModes SelectionMode
        {
            get { return (SelectionModes)GetValue(SelectionModeProperty); }
            set { SetValue(SelectionModeProperty, value); }
        }

        /// <summary>
        /// Handles changes to the SelectionMode property.
        /// </summary>
        private static void OnSelectionModeChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            SelectionPluginBehavior target = (SelectionPluginBehavior)d;
            SelectionModes oldSelectionMode = (SelectionModes)e.OldValue;
            SelectionModes newSelectionMode = target.SelectionMode;
            target.OnSelectionModeChanged(oldSelectionMode, newSelectionMode);
        }

        /// <summary>
        /// Provides derived classes an opportunity to handle changes to the SelectionMode property.
        /// </summary>
        protected virtual void OnSelectionModeChanged(SelectionModes oldSelectionMode, SelectionModes newSelectionMode)
        {
        }

        #endregion

        #region TargetType

        /// <summary>
        /// TargetType Dependency Property
        /// </summary>
        public static readonly DependencyProperty TargetTypeProperty =
            DependencyProperty.Register("TargetType", typeof(Type), typeof(SelectionPluginBehavior),
                new FrameworkPropertyMetadata((Type)typeof(DependencyObject),
                    new PropertyChangedCallback(OnTargetTypeChanged)));

        /// <summary>
        /// Gets or sets the TargetType property. This dependency property 
        /// indicates ....
        /// </summary>
        public Type TargetType
        {
            get { return (Type)GetValue(TargetTypeProperty); }
            set { SetValue(TargetTypeProperty, value); }
        }

        /// <summary>
        /// Handles changes to the TargetType property.
        /// </summary>
        private static void OnTargetTypeChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            SelectionPluginBehavior target = (SelectionPluginBehavior)d;
            Type oldTargetType = (Type)e.OldValue;
            Type newTargetType = target.TargetType;
            target.OnTargetTypeChanged(oldTargetType, newTargetType);
        }

        /// <summary>
        /// Provides derived classes an opportunity to handle changes to the TargetType property.
        /// </summary>
        protected virtual void OnTargetTypeChanged(Type oldTargetType, Type newTargetType)
        {
        }

        #endregion



        #endregion

        #region Selection Handling

        protected void StartSelection(Selection selection)
        {
            AdornerLayer.Add(new SelectionView(AssociatedObject, selection, SelectionMode));
            ParentBehavior.RaiseSelectionStartedEvent(selection);
        }

        protected void ChangeSelection(Selection selection)
        {
            ParentBehavior.RaiseSelectionChangedEvent(selection);
        }

        protected void FinishSelection(Selection selection)
        {
            var selectionView = AdornerLayer.GetAdorners(AssociatedObject).OfType<SelectionView>().Where(view => view.Selection == selection).Single();
            AdornerLayer.Remove(selectionView);

            selection.SelectedElements = new List<DependencyObject>(ZHitTestHelper.GetElementsInGeometry(selection.SelectionGeometry, AssociatedObject, TargetType));

            ParentBehavior.RaiseSelectionFinishedEvent(selection);
        }

        protected void CancelSelection(Selection selection)
        {
            ParentBehavior.RaiseSelectionCancelledEvent(selection);
        }

        #endregion
    }
}
