﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using ZOIL.Framework.Presentation;
using ZOIL.Library.Behaviors.Viewport;
using System.Windows;

namespace ZOIL.Library.Behaviors
{
    public class SendViewportBehavior : RuntimeBehavior<ZInformationLandscape, SendViewportPluginBehavior>
    {
        #region dp

        #region IsEnabled

        /// <summary>
        /// IsEnabled Dependency Property
        /// </summary>
        public static readonly DependencyProperty IsEnabledProperty =
            DependencyProperty.Register("IsEnabled", typeof(bool), typeof(SendViewportBehavior),
                new FrameworkPropertyMetadata((bool)true,
                    new PropertyChangedCallback(OnIsEnabledChanged)));

        /// <summary>
        /// Gets or sets the IsEnabled property. This dependency property 
        /// indicates ....
        /// </summary>
        public bool IsEnabled
        {
            get { return (bool)GetValue(IsEnabledProperty); }
            set { SetValue(IsEnabledProperty, value); }
        }

        /// <summary>
        /// Handles changes to the IsEnabled property.
        /// </summary>
        private static void OnIsEnabledChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            SendViewportBehavior target = (SendViewportBehavior)d;
            bool oldIsEnabled = (bool)e.OldValue;
            bool newIsEnabled = target.IsEnabled;
            target.OnIsEnabledChanged(oldIsEnabled, newIsEnabled);
        }

        /// <summary>
        /// Provides derived classes an opportunity to handle changes to the IsEnabled property.
        /// </summary>
        protected virtual void OnIsEnabledChanged(bool oldIsEnabled, bool newIsEnabled)
        {
        }

        #endregion

        #region SenderID

        /// <summary>
        /// SenderID Dependency Property
        /// </summary>
        public static readonly DependencyProperty SenderIDProperty =
            DependencyProperty.Register("SenderID", typeof(String), typeof(SendViewportBehavior),
                new FrameworkPropertyMetadata((String)null,
                    new PropertyChangedCallback(OnSenderIDChanged)));

        /// <summary>
        /// Gets or sets the SenderID property. This dependency property 
        /// indicates ....
        /// </summary>
        public String SenderID
        {
            get { return (String)GetValue(SenderIDProperty); }
            set { SetValue(SenderIDProperty, value); }
        }

        /// <summary>
        /// Handles changes to the SenderID property.
        /// </summary>
        private static void OnSenderIDChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            SendViewportBehavior target = (SendViewportBehavior)d;
            String oldSenderID = (String)e.OldValue;
            String newSenderID = target.SenderID;
            target.OnSenderIDChanged(oldSenderID, newSenderID);
        }

        /// <summary>
        /// Provides derived classes an opportunity to handle changes to the SenderID property.
        /// </summary>
        protected virtual void OnSenderIDChanged(String oldSenderID, String newSenderID)
        {
        }

        #endregion

        #endregion
    }
}
