﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using ZOIL.Framework.Presentation;
using System.Windows;

namespace ZOIL.Library.Behaviors.Viewport
{
    public class SendViewportPluginBehavior : PluginBehavior<ZInformationLandscape, SendViewportBehavior>
    {
        #region private fields

        private IDisposable _landscapeChangedSubscription;

        #endregion

        #region ctor

        public SendViewportPluginBehavior() : base()
        {

        }

        #endregion

        #region overrides

        protected override void OnParentBehaviorChanged(object oldParentBehavior, object newParentBehavior)
        {
            IsEnabled = ParentBehavior.IsEnabled;
            SenderID = ParentBehavior.SenderID;
            
            CreateOneWayParentBehaviorBinding(SendViewportBehavior.IsEnabledProperty, SendViewportPluginBehavior.IsEnabledProperty);
            CreateOneWayParentBehaviorBinding(SendViewportBehavior.SenderIDProperty, SendViewportPluginBehavior.SenderIDProperty);           
        }

        protected override void OnAttached()
        {
            if (IsEnabled)
                _landscapeChangedSubscription = RegisterLandscapeChangedEvent();
        }

        #endregion

        #region private methods

        private IDisposable RegisterLandscapeChangedEvent()
        {
            var landscapeChanged = Observable.FromEvent<ZLandscapeAnimationEventArgs>(AssociatedObject, "AnimationChanged").Select(evt => AssociatedObject);

            return landscapeChanged.Subscribe(OnViewportChanged);
        }

        #endregion

        #region dp

        #region IsEnabled

        /// <summary>
        /// IsEnabled Dependency Property
        /// </summary>
        public static readonly DependencyProperty IsEnabledProperty =
            DependencyProperty.Register("IsEnabled", typeof(bool), typeof(SendViewportPluginBehavior),
                new FrameworkPropertyMetadata((bool)true,
                    new PropertyChangedCallback(OnIsEnabledChanged)));

        /// <summary>
        /// Gets or sets the IsEnabled property. This dependency property 
        /// indicates ....
        /// </summary>
        public bool IsEnabled
        {
            get { return (bool)GetValue(IsEnabledProperty); }
            set { SetValue(IsEnabledProperty, value); }
        }

        /// <summary>
        /// Handles changes to the IsEnabled property.
        /// </summary>
        private static void OnIsEnabledChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            SendViewportPluginBehavior target = (SendViewportPluginBehavior)d;
            bool oldIsEnabled = (bool)e.OldValue;
            bool newIsEnabled = target.IsEnabled;
            target.OnIsEnabledChanged(oldIsEnabled, newIsEnabled);
        }

        /// <summary>
        /// Provides derived classes an opportunity to handle changes to the IsEnabled property.
        /// </summary>
        protected virtual void OnIsEnabledChanged(bool oldIsEnabled, bool newIsEnabled)
        {
            if (newIsEnabled)
            {
                _landscapeChangedSubscription = RegisterLandscapeChangedEvent();
            }
            else
            {
                if (_landscapeChangedSubscription != null)
                    _landscapeChangedSubscription.Dispose();
            }
        }

        #endregion

        #region SenderID

        /// <summary>
        /// SenderID Dependency Property
        /// </summary>
        public static readonly DependencyProperty SenderIDProperty =
            DependencyProperty.Register("SenderID", typeof(String), typeof(SendViewportPluginBehavior),
                new FrameworkPropertyMetadata((String)null,
                    new PropertyChangedCallback(OnSenderIDChanged)));

        /// <summary>
        /// Gets or sets the SenderID property. This dependency property 
        /// indicates ....
        /// </summary>
        public String SenderID
        {
            get { return (String)GetValue(SenderIDProperty); }
            set { SetValue(SenderIDProperty, value); }
        }

        /// <summary>
        /// Handles changes to the SenderID property.
        /// </summary>
        private static void OnSenderIDChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            SendViewportPluginBehavior target = (SendViewportPluginBehavior)d;
            String oldSenderID = (String)e.OldValue;
            String newSenderID = target.SenderID;
            target.OnSenderIDChanged(oldSenderID, newSenderID);
        }

        /// <summary>
        /// Provides derived classes an opportunity to handle changes to the SenderID property.
        /// </summary>
        protected virtual void OnSenderIDChanged(String oldSenderID, String newSenderID)
        {
        }

        #endregion

        #endregion

        #region virtual methods

        protected virtual void OnViewportChanged(ZInformationLandscape landscape)
        {

        }

        #endregion
    }
}
