﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using ZOIL.DataBackend;
using System.Windows;
using System.Windows.Media;
using System.Collections.ObjectModel;
using System.Globalization;
using System.Threading;

namespace ZOIL.Library.Components
{
    public class ZPortalShape : DInformationObject
    {
        #region ctor

        public ZPortalShape()
            : base()
        {
            IsInitialized = false;
        }

        #endregion

        #region public properties

        #region Path

        private String Path
        {
            get
            {
                return GetValue<String>("zportalshape:path");
            }
            set
            {
                SetValue("zportalshape:path", value);
                RaisePropertyChangedEvent("Path");
            }
        }

        #endregion

        #region HitTestType

        public Type HitTestType
        {
            get
            {
                return GetValue<Type>("zportalshape:hittesttype");
            }
            set
            {
                SetValue("zportalshape:hittesttype", value);
                RaisePropertyChangedEvent("HitTestType");
            }
        }

        #endregion

        #region ShapeGeometry

        public Geometry ShapeGeometry
        {
            get
            {
                PathGeometry shapeGeometry = null;

                //we need to set the current culture to en-US because the parser needs to have the same culture as the toString() method used to save the path
                CultureInfo oldCultureInfo = Thread.CurrentThread.CurrentCulture;
                CultureInfo oldUICultureInfo = Thread.CurrentThread.CurrentUICulture;
                Thread.CurrentThread.CurrentCulture = new CultureInfo("en-US");
                Thread.CurrentThread.CurrentUICulture = Thread.CurrentThread.CurrentCulture;

                shapeGeometry = Geometry.Parse(Path).GetOutlinedPathGeometry();

                //set the culture back to the previous
                Thread.CurrentThread.CurrentCulture = oldCultureInfo;
                Thread.CurrentThread.CurrentUICulture = oldUICultureInfo;

                return shapeGeometry;
            }
            set
            {
                //this is mostly called after a selection has been finished and a new PortalShape was initialized

                PathGeometry shapeGeometry = value.GetOutlinedPathGeometry();
                Path = shapeGeometry.ToString(new CultureInfo("en-US"));
                RaisePropertyChangedEvent("ShapeGeometry");
            }
        }

        #endregion

        #region SelectedElements

        private DObservableCollection<Object> _selectedElements;

        public DObservableCollection<Object> SelectedElements
        {
            get
            {
                if (_selectedElements == null)
                {
                    _selectedElements = new DObservableCollection<Object>();
                }
                return _selectedElements;
            }
            set
            {
                _selectedElements = value;
                RaisePropertyChangedEvent("SelectedElements");
            }
        }

        #endregion

        #region IsInitialized

        public bool IsInitialized
        {
            get
            {
                return (bool)GetValue("zportalshape:isinitialized");
            }
            set
            {
                SetValue("zportalshape:isinitialized", value);
                RaisePropertyChangedEvent("IsInitialized");
            }
        }

        #endregion

        #region Bounds

        public Rect Bounds
        {
            get
            {
                return ShapeGeometry.Bounds;
            }
        }

        #endregion

        #endregion

        #region public methods

        public void InvalidateShape()
        {
            this.ShapeGeometry = ShapeGeometry;
        }

        #endregion
    }
}
