﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using ZOIL.DataModel;
using System.Windows.Input;
using System.Windows;
using ZOIL.Framework.Presentation;
using Microsoft.Surface.Presentation.Controls;
using System.Windows.Controls;
using ZOIL.Framework.Core;

namespace ZOIL.Library.Components
{
    [ViewModelFor(typeof(ZPhotoView))]
    public class ZPhotoViewModel : BaseViewModel
    {

        public ZPhotoViewModel()
            : base()
        {
            RegisterChangeNotification("Title", "Title");
            RegisterChangeNotification("Tags", "Tags");
            RegisterChangeNotification("Description", "Description");
            RegisterChangeNotification("UserName", "UserName");
            RegisterChangeNotification("DateUploaded", "DateUploaded");
            RegisterChangeNotification("SmallUrl", "SmallUrl");
            RegisterChangeNotification("PhotoId", "PhotoId");
            RegisterChangeNotification("BuddyIconURL", "BuddyIconURL");
            RegisterChangeNotification("Views", "Views");


            ZoomCommand= new RoutedCommand("ZoomCommand", typeof(ZPhotoViewModel));
            CommandBinding binding = new CommandBinding(ZoomCommand, ExecuteZoomCommand, CanExecuteZoomCommand);
            this.CommandBindings.Add(binding);

        }

        public new ZPhoto Model
        {
            get
            {
                return base.Model as ZPhoto;
            }
            set
            {
                base.Model = value;
            }
        }

        #region Properties

        public String Title
        {
            get
            {
                return (Model as ZPhoto).Title;
            }
            set
            {
                (Model as ZPhoto).Title = value;
                RaisePropertyChangedEvent("Title");
            }
        }

        public IList<String> Tags
        {
            get
            {
                return (Model as ZPhoto).Tags;
            }
            set
            {
                (Model as ZPhoto).Tags = value;
                RaisePropertyChangedEvent("Tags");
            }
        }

        public String Description
        {
            get
            {
                return (Model as ZPhoto).Description;
            }
            set
            {
                (Model as ZPhoto).Description = value;
                RaisePropertyChangedEvent("Description");
            }
        }

        public String UserName
        {
            get
            {
                return (Model as ZPhoto).UserName;
            }
            set
            {
                (Model as ZPhoto).UserName = value;
                RaisePropertyChangedEvent("UserName");
            }
        }

        public DateTime DateUploaded
        {
            get
            {
                return (Model as ZPhoto).DateUploaded;
            }
            set
            {
                (Model as ZPhoto).DateUploaded = value;
                RaisePropertyChangedEvent("DateUploaded");
            }
        }

        public Uri BuddyIconURL
        {
            get
            {
                return (Model as ZPhoto).BuddyIconURL;
            }
            set
            {
                (Model as ZPhoto).BuddyIconURL = value;
                RaisePropertyChangedEvent("BuddyIconURL");
            }
        }

        public int Views
        {
            get
            {
                return (Model as ZPhoto).Views;
            }
            set
            {
                (Model as ZPhoto).Views = value;
                RaisePropertyChangedEvent("Views");
            }
        }

        public String PhotoId
        {
            get
            {
                return (Model as ZPhoto).PhotoId;
            }
            set
            {
                (Model as ZPhoto).PhotoId = value;
                RaisePropertyChangedEvent("PhotoId");
            }
        }

        public Uri SmallUrl
        {
            get
            {
                return new Uri((Model as ZPhoto).SmallUrl.ToString());
            }
            set
            {
                (Model as ZPhoto).SmallUrl = value;
                RaisePropertyChangedEvent("SmallUrl");
            }
        }
        #endregion

        #region Commands

        #region ZoomCommand

        private RoutedCommand _zoomCommand;
        public RoutedCommand ZoomCommand
        {
            get
            {
                return _zoomCommand;
            }
            set
            {
                _zoomCommand = value;
                RaisePropertyChangedEvent("ZoomCommand");
            }
        }

        private void ExecuteZoomCommand(object sender, ExecutedRoutedEventArgs e)
        {

            
            String oscAdress = "none";
            int cubePort = 0;

            #region identify cube

            SurfaceButton b = e.OriginalSource as SurfaceButton;
            if (b != null)
            {
                if (b.Name == "Cube1Button")
                {
                    oscAdress = "/cube1/landscape";
                    cubePort = 1;
                }
                else if (b.Name == "Cube2Button")
                {
                    oscAdress = "/cube2/landscape";
                    cubePort = 2;
                }
            }
            #endregion

            //identify the clicked Element
            FrameworkElement target = this.View as FrameworkElement;
            if (target != null)
            {
                //identify the next zoom target in the visual tree
                target = ZTreeHelper.GetClosestZoomTarget(target);
                Panel.SetZIndex(target, ZComponent.ZIndexCounter++);

                //get landscape to calculate the bounds of the viewports
                ZInformationLandscape zil = ZTreeHelper.TryFindParent<ZInformationLandscape>(target);
                if (zil == null)
                {
                    return;
                }
                Rect rect = zil.GetControlPositionInLandscape(target);

                //send the calculated rectangle via OSC to the cube (part of the multicast group)
                //OSCSender.sendZoomCommand(System.Net.IPAddress.Parse("224.5.6.7"), oscAdress , 4497 + cubePort,
                //    rect.X,
                //    rect.Y,
                //    rect.Width,
                //    rect.Height, 1);
            }

        }

        private void CanExecuteZoomCommand(object sender, CanExecuteRoutedEventArgs e)
        {
            e.CanExecute = true;
        }

        #endregion

        #endregion

    }
}
