﻿using System.Windows;
using System.Windows.Input;
using ZOIL.Framework.Presentation;
using ZOIL.Framework.Core;

namespace ZOIL.Library.Devices
{
    /// <summary>
    /// Default Handler for the Keyboard Device
    /// </summary>
    public class ZLandscapeDefaultKeyboardHandler : ZLandscapeInputHandler
    {
        #region ctor

        /// <summary>
        /// Constructor of the default handler for the keyboard device
        /// </summary>
        /// <param name="landscape"></param>
        public ZLandscapeDefaultKeyboardHandler(ZInformationLandscape landscape) : base(landscape)
        {
            IsEnabled = true;
        }

        #endregion

        #region KeyDownHandler

        private void OnKeyDown(object sender, KeyEventArgs e)
        {
            // Keyboard events are only processed for this.Landscape, if ...
            // 1. mouse pointer is inside the landscape
            // 2. and the landscape is the "deepest" one at this mouse position down the visual tree
            if (!this.InformationLandscape.IsMouseOver || (ZTreeHelper.TryFindParent<ZInformationLandscape>((DependencyObject)Mouse.DirectlyOver) != this.InformationLandscape))
                return;

            switch (e.Key)
            {
                // Cursor down means zoom out
                case Key.Down:
                    {
                        // Zoom out of landscape (with zooming ref point at mouse position)
                        this.InformationLandscape.ZoomBy(Mouse.GetPosition(this.InformationLandscape), 0.7, true);  
                        break;
                    }

                // Cursor up means zoom in
                case Key.Up:
                    {
                        // Zoom into landscape (with zooming ref point at mouse position)
                        this.InformationLandscape.ZoomBy(Mouse.GetPosition(this.InformationLandscape), 1.0 / 0.7, true);
                        break;
                    }
            }
        }

        #endregion

        #region overrides

        /// <summary>
        /// called whenever the value of <see cref="ZLandscapeInputHandler.IsZoomingEnabled"/> has changed
        /// </summary>
        protected override void OnIsZoomingEnabledChanged()
        {
            if (IsZoomingEnabled)
                Keyboard.AddKeyDownHandler(CurrentWindow, OnKeyDown);
            else
                Keyboard.RemoveKeyDownHandler(CurrentWindow, OnKeyDown);
        }

        #endregion
    }
}
