﻿using System;
using System.Windows;
using System.Windows.Interactivity;

namespace ZOIL.Library.Behaviors
{
    public class RoutedEventTrigger : EventTriggerBase<DependencyObject>
    {
        public RoutedEvent RoutedEvent { get; set; }

        protected override void OnAttached()
        {
            var behavior = AssociatedObject as Behavior;
            var associatedElement = AssociatedObject as FrameworkElement;

            if (behavior != null)
            {
                associatedElement = ((IAttachedObject)behavior).AssociatedObject as FrameworkElement;
            }

            if (associatedElement == null)
            {
                throw new ArgumentException("Routed Event trigger can only be associated to framework elements");
            }
            if (RoutedEvent != null)
            {
                associatedElement.AddHandler(RoutedEvent, new RoutedEventHandler(OnRoutedEvent));
            }
        }
        protected virtual void OnRoutedEvent(object sender, RoutedEventArgs args)
        {
            base.OnEvent(args);
        }
        protected override string GetEventName()
        {
            return RoutedEvent.Name;
        }
    }

    public class RoutedEventTriggerAdvanced : RoutedEventTrigger
    {
        protected override void OnRoutedEvent(object sender, RoutedEventArgs args)
        {
            base.OnEvent(new SenderAwareEventArgs {Sender = sender, OriginalEventArgs = args});
        }
    }

    public class SenderAwareEventArgs : EventArgs
    {
        public object Sender { get; set; }
        public RoutedEventArgs OriginalEventArgs { get; set; }
    }
}